<?php
if (!defined('MEDIAWIKI')) die("MediaWiki extensions cannot be run directly.");
if (!defined('WikiDB')) {
	die("The WikiDB extension has not been set up correctly.<br>'"
		. basename(__FILE__) . "' should not be included directly.<br>Instead, "
		. "add the following to LocalSettings.php: "
		. "<tt><b>include_once(\"WikiDB/WikiDB.php\");</b></tt>");
}
/**
 * WikiDB Parser class.
 * A set of static functions used to parse the various forms of input data into
 * a format that we can work with.
 *
 * @package MediaWiki
 * @subpackage Extensions
 *
 * @author Mark Clements <mclements at kennel17 dot co dot uk>
 * @copyright Copyright  2006-2010, Mark Clements
 * @license http://creativecommons.org/licenses/by-sa/2.5/ cc-by-sa 2.5 or later
 * @version $Rev: 466 $
 */

WikiDB_RegisterRevision('$Rev: 466 $');

class WikiDB_Parser {

	function WikiDB_Parser() {
	// Never instantiate this object!
	}

// ParseTableDef()
//
// Takes an input string and returns an array containing any fields it defines.
// Each field is on a single line, and any line that does not match the correct
// format for a field definition is ignored.  See pParseFieldDef() for details about
// the field definition.
// The returned array is formatted as key/value pairs.  For each definition line
// found, key is the name of the field and the value is an array holding the field
// definition, as returned from pParseFieldDef() (see below).  If there was other
// text that occurred between the definition lines then these are indexed as 'textN'
// where N begins with 0.  Multiple text lines are grouped together.
// If $IncludeOtherText is false, then the text lines are not returned.
	function ParseTableDef($Input, $IncludeOtherText = true) {
		$Lines = explode("\n", $Input);

		$Result = array();
		$TextCounter = 0;
		$Text = "";
		$TableDef = array();
		$InComment = false;

		foreach ($Lines as $Line) {
			$FieldDef = WikiDB_Parser::pParseFieldDef($Line);

		// If the line was a field definition, then add it to the array.
		// Before doing that, if there was any text found, and
		// $IncludeOtherText = true, then add the text and reset the text variables.
			if ($FieldDef && !$InComment) {
				if ($IncludeOtherText && $Text != "") {
					$TableDef['text' . $TextCounter] = $Text;
					$Text = "";
					$TextCounter++;
				}
				$TableDef[$FieldDef['FieldName']] = $FieldDef;
			}
		// Otherwise, add the non-definition line to the text, ready to be added to
		// the array as necessary.
			else {
				$Text .= $Line . "\n";
				$InComment = WikiDB_Parser::pResolveCommentState($Line, $InComment);
			}
		}

	// Add any left-over text from the bottom of the page.
		if ($IncludeOtherText && $Text != "")
			$TableDef['text' . $TextCounter] = $Text;

		return $TableDef;
	}

	function pResolveCommentState($Line, $InComment) {
		if ($InComment) {
			$Pos = strpos($Line, "-->");
			if ($Pos === false)
				return $InComment;
			$Pos += 3;
		}
		else {
			$Pos = strpos($Line, "<!--");
			if ($Pos === false)
				return $InComment;
			$Pos += 4;
		}
		
		$Line = substr($Line, $Pos);
		$InComment = !$InComment;
		return WikiDB_Parser::pResolveCommentState($Line, $InComment);
	}

// pParseFieldDef()
//
// Takes a single line of text and checks whether it is a valid field definition.
// If so then an array containing details about the field definition (see below)
// is returned. If not then the function returns NULL.
//
// A valid field definition has the following components.  Only the first two are
// required. Any of the others may be omitted, but if present they must be in the
// order specified.  Whitespace is allowed pretty much anywhere.  In practice, this
// means that any line that begins with a greater-than symbol and has one or more
// non-whitespace characters is a valid field definition.
// TODO: Currently lines that start with > and just contain whitespace (or nothing
//		 else) are treated as normal text lines, whereas actually they should
//		 perhaps be treated as invalid field definitions.
//
//	> 		   - (greater than symbol) - must be the first character on the line.
//	FieldName  - must not contain ":", "[" or "-"
//	:Type	   - must not contain spaces, "(", "[" or "-".  Initial colon is not part
//			     of the type name.
//	(Args)	   - Args is a comma-separated list of arguments, which are
//			     type-specific.  Only allowed if a type is specified.
//	[[#Alias]] - The name of another field, which this field is an alias for.
//	-- Comment - Anything else you like, provided it is separated by a double-hyphen.
//
// e.g.
//	> MyField:Integer (1, 10) -- This field holds an integer between 1 and 10.
//	> MyAlias [[#MyField]] -- This field is an alias for MyField.
//
// If successful, the returned array will have whichever of the following elements
// were defined:
//	'FieldName' => The name of the field (truncated to 255 characters)
//	'Type'		=> The data type of the field (as entered - it may not be a valid
//				   type).
//	'Options'	=> A numerically-indexed array of type arguments.  These are all
//				   strings (no conversion is performed).
//	'Alias'		=> The field that this is an alias for (truncated to 255 characters).
//	'Comment'	=> The comment (not including the double-hyphen).
//
// 'FieldName' is always present.
// 'Options' is only present if 'Type' is present and options were defined.
// 'Alias' or 'Type' will not both be present.  It will be one or other of them
// (or neither).
// 'Comment' is only present if a comment was added.
	function pParseFieldDef($Line) {
		global $pRegexDBLine;

		$FieldData = array();

		if (preg_match($pRegexDBLine, $Line, $Match)) {
		// $Match contains the following elements:
		// [0] - Whole line
		// [1] - Field name
		// [2] - Field type
		// [3] - Arguments (comma-separated list)
		// [4] - Link (if it starts with # it's an alias) - square brackets not
		//		 included.
		// [5] - Comment (not including starting dashes) - i.e. rest of the line.

		// The field name is always present.  We trim() it and truncate it to 255
		// characters.
		// TODO: Any other safety checks required?
			$FieldData['FieldName'] = substr(trim($Match[1]), 0, 255);

		// If the 4th argument is present, it is either an alias or a foreign key
		// reference.
			if (!empty($Match[4])) {
				$Match[4] = trim($Match[4]);
			// If it begins with # then it is an alias for another field, in which
			// case we trim the # and truncate to 255 characters, and store it in
			// the alias field.
			// TODO: Check if third arg to substr() is length or character offset
			//		 (i.e. whether it should be 255 chars, or position 256).
				if ($Match[4]{0} == "#")
					$FieldData['Alias'] = substr($Match[4], 1, 255);

		/* Foreign keys disabled, as there is currently no functionality associated
		   with them.
			// Otherwise it is a foreign key
				else {
					$ForeignKey = explode("#", $Match[4]);
					$FieldData['ForeignTable'] = $ForeignKey[0];
					if (isset($ForeignKey[1]))
						$FieldData['ForeignField'] = $ForeignKey[1];
				// TODO: Decide whether to populate 'ForeignField' with a the correct
				// 		 name if none is specified, or whether to leave it undefined
				//		 and let the resolution of this be handled by other
				//		 functions.
//					else
//						$FieldData['ForeignField'] = $FieldData['FieldName'];
				}
		*/

			}
		// Fill in the type info, if available.  This is ignored for aliases.
			if (!isset($FieldData['Alias'])) {
				if (!empty($Match[2])) {
					$FieldData['Type'] = strtolower(trim($Match[2]));

				// Options are only valid if a type was specified.
					if (!empty($Match[3])) {
						$FieldData['Options'] = explode(",", $Match[3]);
						if (count($FieldData['Options']) > 0) {
							foreach ($FieldData['Options'] as $Key => $Option)
								$FieldData['Options'][$Key] = trim($Option);
						}
					}
				}
			}

		// If there is a comment string, add it to the array
			if (!empty($Match[5]))
				$FieldData['Comment'] = trim($Match[5]);

			return $FieldData;
		}
		else
			return NULL;
	}

// ParseDataTag()
//
// Takes the contents of a <data> tag and returns an array where each 
// numerically-indexed element defines a row of data (destined for the same table),
// represented as an array of FieldName => Value pairs.
// All values in the returned array are strings, and all field names and values are
// trim()med before being returned.
// There are two parse methods, depending on whether the $FieldList argument is set
// or not.
// If $FieldList is NULL then the data is in single-row mode, which means that only 
// a single row will be returned.  It parses each row of the $Input string and any 
// which contains an = sign are treated as a FieldName=Value pair (if there is more
// than one = sign then the first is the separator between field name and value).
// Blank lines or lines without an equals sign are treated as comments and ignored.
// Alternatively, if $FieldList is not NULL, then we are in multi-row mode.  In this 
// case, $FieldList is a list of field names separated by commas and each line of the
// $Input string defines a separate row of table data (unless it is blank or a
// MediaWiki header (at any level), e.g. "== Header ==".  These lines are ignored).
// Each non-ignored row in the $Input string consists of a set of field values, 
// separated by the specified $Separator string (by default, a comma).  The fields
// are in the order specified by $FieldList, and if there are more items in the row 
// than there are in $FieldList, the extra fields are ignored.  If there are fewer 
// then the missing fields are set to "".
	function ParseDataTag($Input, $FieldList = NULL, $Separator = ",") {
		$Lines = explode("\n", $Input);

		$Data = array();

	// Single-row mode:
		if ($FieldList === NULL) {
			foreach ($Lines as $Line) {
			// TODO: Are the /sm modifiers needed?  If so, document them!
				if (preg_match('/^([^=]+)=(.*)$/sm', $Line, $ParsedLine))
					$Data[0][trim($ParsedLine[1])] = trim($ParsedLine[2]);
			}
		}

	// Multi-row mode:
		else {
			$FieldNames = explode(",", $FieldList);
			foreach ($FieldNames as $Key => $Value)
				$FieldNames[$Key] = trim($Value);

			$Row = 0;
			foreach ($Lines as $Line) {
			// Skip blank lines
				if (trim($Line) == "")
					continue;

			// Skip 'headers'
			// Regex based on the definition in parser::doHeadings() in MW 1.6.10
				if (preg_match('/^(={1,6})(.+)\\1\\s*$/', $Line))
					continue;

			// Otherwise the row contains data, so we add it to the output
			// array.
				$FieldValues = explode($Separator, $Line);
				foreach ($FieldNames as $FieldIndex => $FieldName) {
					if (isset($FieldValues[$FieldIndex]))
						$FieldValue = trim($FieldValues[$FieldIndex]);
					else
						$FieldValue = "";

					$Data[$Row][$FieldName] = $FieldValue;
				}

			// Increment the row counter.
				$Row++;
			}
		}

		return $Data;
	}

// ParseQueryTag()
//
// Takes the contents of the query tag and returns each section that it matches,
// indexed by the header name.  It only parses headers it recognises, so other
// headers are treated as standard wiki text in the appropriate manner.
// TODO: Not currently used, as syntax not decided.
	function ParseQueryTag($Input) {
	}

} // END: class WikiDB_Parser

